@@ -87,9 +87,15 @@ namespace eval ::crc {
     # calculate the sign bit for the current platform.
     variable signbit
     if {![info exists signbit]} {
-	variable v
-        for {set v 1} {int($v) != 0} {set signbit $v; set v [expr {$v<<1}]} {}
-        unset v
+        if {[info exists tcl_platform(wordSize)]} {
+            set signbit [expr {1 << (8*$tcl_platform(wordSize)-1)}]
+        } else {
+            # Old Tcl. Find bit by shifting until wrap around to 0.
+            # With int() result limited to system word size the loop will end.
+            variable v
+            for {set v 1} {int($v) != 0} {set signbit $v; set v [expr {$v<<1}]} {}
+            unset v
+        }
     }
     
     variable uid ; if {![info exists uid]} {set uid 0}
@@ -367,7 +373,7 @@ namespace eval ::crc {
     unset e
 }
 
-package provide crc32 1.3.2
+package provide crc32 1.3.3
 
 # -------------------------------------------------------------------------
 #
