@@ -10,45 +10,37 @@ namespace eval ::tcllib::testutils {
 }
 
 # ### ### ### ######### ######### #########
-## Commands for common functions and boilerplate actions required by
-## many testsuites of Tcllib modules and packages in a central place
-## for easier maintenance.
+## Procedures for common functions and boilerplate actions required by many
+## test suites of Tcllib modules and packages.
 
+
 # ### ### ### ######### ######### #########
-## Declare the minimal version of Tcl required to run the package
-## tested by this testsuite, and its dependencies.
+## Declares the minimal version of Tcl and the dependencies required by the
+## package tested by this test suite.  Must be called immediately after loading
+## the utilities.  Bails out of the calling level if the required minimum
+## version is not met by the active interpreter.
 
 proc testsNeedTcl {version} {
-    # This command ensures that a minimum version of Tcl is used to
-    # run the tests in the calling testsuite. If the minimum is not
-    # met by the active interpreter we forcibly bail out of the
-    # testsuite calling the command. The command has to be called
-    # immediately after loading the utilities.
-
     if {[package vsatisfies [package provide Tcl] $version]} return
 
     puts "    Aborting the tests found in \"[file tail [info script]]\""
     puts "    Requiring at least Tcl $version, have [package present Tcl]."
 
-    # This causes a 'return' in the calling scope.
+    # Effect a 'return' at the caller's level.
     return -code return
 }
 
+
 # ### ### ### ######### ######### #########
-## Declare the minimum version of Tcltest required to run the
-## testsuite.
+## Declares the minimum version of Tcltest required to run the test suite.
+## Must be called after loading the utilities.  Loads a suitable version of The
+## only procedure that may preceed it is 'testNeedTcl' above.  Tcltest if the
+## package has not been loaded yet.  Bail out of the test script that called
+## this procedure if the loaded version of tcltest does not meet the given
+## minimum version,
 
 proc testsNeedTcltest {version} {
-    # This command ensure that a minimum version of the Tcltest
-    # support package is used to run the tests in the calling
-    # testsuite. If the minimum is not met by the loaded package we
-    # forcibly bail out of the testsuite calling the command. The
-    # command has to be called after loading the utilities. The only
-    # command allowed to come before it is 'testNeedTcl' above.
-
-    # Note that this command will try to load a suitable version of
-    # Tcltest if the package has not been loaded yet.
-
+    regexp {^([^-]*)} $version -> minversion
     if {[lsearch [namespace children] ::tcltest] == -1} {
 	if {![catch {
 	    package require tcltest $version
@@ -57,7 +49,7 @@ proc testsNeedTcltest {version} {
 	    InitializeTclTest
 	    return
 	}
-    } elseif {[package vcompare [package present tcltest] $version] >= 0} {
+    } elseif {[package vcompare [package present tcltest] $minversion] >= 0} {
 	InitializeTclTest
 	return
     }
@@ -65,16 +57,15 @@ proc testsNeedTcltest {version} {
     puts "    Aborting the tests found in [file tail [info script]]."
     puts "    Requiring at least tcltest $version, have [package present tcltest]"
 
-    # This causes a 'return' in the calling scope.
+    # Effect a return at the level of the caller.
     return -code return
 }
 
 proc testsNeed {name {version {}}} {
-    # This command ensures that a minimum version of package <name> is
-    # used to run the tests in the calling testsuite. If the minimum
-    # is not met by the active interpreter we forcibly bail out of the
-    # testsuite calling the command. The command has to be called
-    # immediately after loading the utilities.
+    # Must be called immediately after loading the utilities.  Loads the named
+    # package if it is not already loaded.  If the version of the loaded
+    # package does not meet the given minimum version, bail out of the test
+    # suite that called the procedure.
 
     if {$version != {}} {
 	if {[catch {
@@ -82,7 +73,7 @@ proc testsNeed {name {version {}}} {
 	}]} {
 	    puts "    Aborting the tests found in \"[file tail [info script]]\""
 	    puts "    Requiring at least \"$name $version\", package not found."
-	    
+
 	    return -code return
 	}
 
@@ -99,7 +90,7 @@ proc testsNeed {name {version {}}} {
 	}]} {
 	    puts "    Aborting the tests found in \"[file tail [info script]]\""
 	    puts "    Requiring \"$name\", package not found."
-	    
+
 	    return -code return
 	}
     }
@@ -107,20 +98,17 @@ proc testsNeed {name {version {}}} {
 
 # ### ### ### ######### ######### #########
 
-## Save/restore the environment, for testsuites which have to
-## manipulate it to (1) either achieve the effects they test
-## for/against, or (2) to shield themselves against manipulation by
-## the environment. We have examples for both in 'fileutil' (1), and
-## 'doctools' (2).
+## Saves/restores the environment for test suites which manipulate it either to
+## achieve the effects they test for/against, or to shield themselves against
+## manipulation by the environment.  'fileutil' is an example of the first, and
+## 'doctools' is an example of the second.
 ##
-## Saving is done automatically at the beginning of a test file,
-## through this module. Restoration is done semi-automatically.  We
-## __cannot__ hook into the tcltest cleanup hook It is already used by
-## all.tcl to transfer the information from the slave doing the actual
-## tests to the master. Here the hook is only an alias, and
-## unmodifiable. We create a new cleanup command which runs both our
-## environment cleanup, and the regular one. All .test files are
-## modified to use the new cleanup.
+## The environment is automatically saved at the beginning of a test file, and
+## restoration is semi-automatic.  The tcltest cleanup hook is an unmodifiable
+## alias used by all.tcl to transfer results from the slave iterpreter running
+## the tests to the master interpreter, so create instead a new cleanup
+## command which runs both our environment cleanup and the regular one. All
+## .test files are modified to use the new cleanup.
 
 proc ::tcllib::testutils::SaveEnvironment {} {
     global env
@@ -153,12 +141,11 @@ proc array_unset {a {pattern *}} {
 }
 
 # ### ### ### ######### ######### #########
-## Newer versions of the Tcltest support package for testsuite provide
-## various features which make the creation and maintenance of
-## testsuites much easier. I consider it important to have these
-## features even if an older version of Tcltest is loaded. To this end
-## we now provide emulations and implementations, conditional on the
-## version of Tcltest found to be active.
+## Newer versions of Tcltest provide various features which make it easier to
+## create and maintain a test suite.  I consider it important to have these
+## features even if an older version of Tcltest is loaded, so we now provide
+## emulations and implementations for versions that are missing this
+## functionality.
 
 # ### ### ### ######### ######### #########
 ## Easy definition and initialization of test constraints.
@@ -175,13 +162,13 @@ proc InitializeTclTest {} {
 	}
 	return -code error "No result available. Failed to match any of the constraints ([join [lsort -dict [dict keys $dict]] ,])."
     }
-    
+
     if {![package vsatisfies [package provide tcltest] 2.0]} {
 	# Tcltest 2.0+ provides a documented public API to define and
-	# initialize a test constraint. For earlier versions of the
-	# package the user has to directly set a non-public undocumented
-	# variable in the package's namespace. We create a command doing
-	# this and emulating the public API.
+	# initialize a test constraint. For earlier versions the user has to
+	# directly set a non-public undocumented variable in the package's
+	# namespace.  The following procedures do this, adhering the public
+	# API.
 
 	proc ::tcltest::testConstraint {c args} {
 	    variable testConstraints
@@ -229,6 +216,10 @@ proc InitializeTclTest {} {
     ::tcltest::testConstraint tcl8.6plus \
 	[expr {[package vsatisfies [package provide Tcl] 8.6]}]
 
+    ::tcltest::testConstraint tcl8.6not8.7 \
+	[expr { [package vsatisfies [package provide Tcl] 8.6] &&
+	       ![package vsatisfies [package provide Tcl] 8.7]}]
+
     ::tcltest::testConstraint tcl8.6not10 \
 	[expr { [package vsatisfies [package provide Tcl] 8.6] &&
 	       ![package vsatisfies [package provide Tcl] 8.6.10]}]
@@ -242,6 +233,9 @@ proc InitializeTclTest {} {
     ::tcltest::testConstraint tcl8.5minus \
 	[expr {![package vsatisfies [package provide Tcl] 8.6]}]
 
+    ::tcltest::testConstraint tcl8.7plus \
+	[expr {[package vsatisfies [package provide Tcl] 8.7]}]
+
     # ### ### ### ######### ######### #########
     ## Cross-version code for the generation of the error messages created
     ## by Tcl procedures when called with the wrong number of arguments,
@@ -282,7 +276,7 @@ proc InitializeTclTest {} {
 	}
 
 	proc ::tcltest::tooManyArgs {functionName argList} {
-	    # create a different message for functions with no args
+	    # Create a different message for functions with no args.
 	    if {[llength $argList]} {
 		if {[string match args [lindex $argList end]]} {
 		    set argList [lreplace $argList end end ...]
@@ -302,7 +296,7 @@ proc InitializeTclTest {} {
 	}
 
 	proc ::tcltest::tooManyArgs {functionName argList} {
-	    # create a different message for functions with no args
+	    # Create a different message for functions with no args.
 	    if {[llength $argList]} {
 		set msg "wrong # args: should be \"$functionName $argList\""
 	    } else {
@@ -332,14 +326,14 @@ proc InitializeTclTest {} {
 
 	# The 'makeFile' in Tcltest 1.0 returns a list of all the
 	# paths generated so far, whereas the 'makeFile' in 2.0+
-	# returns only the path of the newly generated file. We
+	# returns only the path of the newly-generated file. We
 	# standardize on the more useful behaviour of 2.0+. If 1.x is
-	# present we have to create an emulation layer to get the
-	# wanted result.
+	# present we create an emulation layer to get the
+	# desired result.
 
-	# 1.0 is not fully correctly described. If the file was
-	# created before no list is returned at all. We force things
-	# by adding a line to the old procedure which makes the result
+	# 1.0 is not fully described correctly. If the file was
+	# created before, no list is returned at all. Force things
+	# here by adding a line to the old procedure which makes the result
 	# unconditional (the name of the file/dir created).
 
 	# The same change applies to 'makeDirectory'
@@ -391,7 +385,7 @@ proc InitializeTclTest {} {
 }
 
 # ### ### ### ######### ######### #########
-## Command to construct wrong/args messages for Snit methods.
+## Constructs wrong/args messages for Snit methods.
 
 proc snitErrors {} {
     if {[package vsatisfies [package provide snit] 2]} {
@@ -410,39 +404,60 @@ proc snitErrors {} {
     } else {
 	proc snitWrongNumArgs {obj method arglist missingIndex} {
 	    incr missingIndex 4
-	    tcltest::wrongNumArgs "$method" [linsert $arglist 0 \
+	    tcltest::wrongNumArgs $method [linsert $arglist 0 \
 		    type selfns win self] $missingIndex
 	}
 
 	proc snitTooManyArgs {obj method arglist} {
-	    tcltest::tooManyArgs "$method" [linsert $arglist 0 \
+	    tcltest::tooManyArgs $method [linsert $arglist 0 \
 		    type selfns win self]
 	}
     }
 }
 
 # ### ### ### ######### ######### #########
-## Commands to load files from various locations within the local
-## Tcllib, and the loading of local Tcllib packages. None of them goes
-## through the auto-loader, nor the regular package management, to
-## avoid contamination of the testsuite by packages and code outside
-## of the Tcllib under test.
+## Procedures that load files from various locations within the local Tcllib
+## or that load local Tcllib packages.  To avoid contamination of the test
+## suite by packages and code outside of the Tcllib under test, none of them go
+## through the auto-loader nor use the regular package management procedures.
 
-# Shorthand for access to module-local assets files for tests.
-proc asset {path} {
-    file join $::tcltest::testsDirectory test-assets $path
+proc asset args {
+    set localPath [file join [uplevel 1 [
+		list [namespace which localPath]]]]
+	foreach location {test-assets {.. test-assets}} {
+		set candidate [eval file join [list $localPath] $location $args]
+		if {[file exists $candidate]} {
+			set {asset path} $candidate
+			break
+		}
+	}
+	if {![info exists {asset path}]} {
+		error [list {can not find asset path}]
+	}
+	return ${asset path}
 }
 
-proc asset-get {path} {
-    set c [open [asset $path] r]
+proc asset-get args {
+	file-get [uplevel 1 [list [namespace which asset]] $args]
+}
+
+proc file-get path {
+    set c [open $path r]
     set d [read $c]
     close $c
     return $d
 }
 
+proc localDirectory {} {
+    set script [uplevel 1 [list ::info script]]
+    file dirname [file dirname [file normalize [
+	    file join $script[set script {}] ...]]]
+}
+
 # General access to module-local files
-proc localPath {fname} {
-    return [file join $::tcltest::testsDirectory $fname]
+proc localPath args {
+    set {script dir} [uplevel 1 [list [namespace which localDirectory]]]
+    eval file join [list ${script dir}] $args
 }
 
 # General access to global (project-local) files
@@ -486,7 +501,7 @@ proc useKeep {fname pname args} {
     # Keep = Keep the existing namespace of the package.
     #      = Do not delete it. This is required if the
     #        namespace contains commands created by a
-    #        binary package, like 'tcllibc'. They cannot
+    #        binary package, like 'tcllibc', as they cannot
     #        be re-created.
     ##
     ## catch {namespace delete $nsname}
@@ -531,7 +546,7 @@ proc useLocalKeep {fname pname args} {
     # Keep = Keep the existing namespace of the package.
     #      = Do not delete it. This is required if the
     #        namespace contains commands created by a
-    #        binary package, like 'tcllibc'. They cannot
+    #        binary package, like 'tcllibc', as they cannot
     #        be re-created.
     ##
     ## catch {namespace delete $nsname}
@@ -569,7 +584,8 @@ proc support {script} {
 
 proc testing {script} {
     InitializeTclTest
-    set ::tcllib::testutils::tag "*"
+
+    set ::tcllib::testutils::tag *
     if {[catch {
 	uplevel 1 $script
     } msg]} {
@@ -612,11 +628,9 @@ proc useTcllibC {} {
 
 # - dictsort -
 #
-#  Sort a dictionary by its keys. I.e. reorder the contents of the
-#  dictionary so that in its list representation the keys are found in
-#  ascending alphabetical order. In other words, this command creates
-#  a canonical list representation of the input dictionary, suitable
-#  for direct comparison.
+#  Sorts a dictionary by its keys so that in its list representation the keys
+#  are found in ascending alphabetical order, making it easier to directly
+#  compare another dictionary
 #
 # Arguments:
 #	dict:	The dictionary to sort.
@@ -634,16 +648,16 @@ proc dictsort {dict} {
 }
 
 # ### ### ### ######### ######### #########
-## Putting strings together, if they cannot be expressed easily as one
-## string due to quoting problems.
+## Puts strings together.  Useful when the strings cannot be expressed easily as
+## one string due to quoting problems.
 
 proc cat {args} {
     return [join $args ""]
 }
 
 # ### ### ### ######### ######### #########
-## Mini-logging facility, can also be viewed as an accumulator for
-## complex results.
+## Mini-logging facility.  Can also be viewed as an accumulator for complex
+## results.
 #
 # res!      : clear accumulator.
 # res+      : add arguments to accumulator.
@@ -672,12 +686,10 @@ proc res?lines {} {
 }
 
 # ### ### ### ######### ######### #########
-## Helper commands to deal with packages
-## which have multiple implementations, i.e.
-## their pure Tcl base line and one or more
-## accelerators. We are assuming a specific
-## API for accessing the data about available
-## accelerators, switching between them, etc.
+## Procedures that help deal with packages that have multiple implementations,
+## i.e.  their pure Tcl implementation along with one or more accelerators.
+## Assumes a specific API for accessing the data about available accelerators,
+## switching between them, etc.
 
 # == Assumed API ==
 #
@@ -686,20 +698,20 @@ proc res?lines {} {
 #
 # Implementations --
 #   Returns list of activated implementations.
-#   A subset of 'KnownImplementations'
+#   A subset of 'KnownImplementations'.
 #
 # Names --
-#   Returns dict mapping all known implementations
+#   Returns a dict mapping all known implementations
 #   to human-readable strings for output during a
-#   test run
+#   test run.
 #
 # LoadAccelerator accel --
 #   Tries to make the implementation named
-#   'accel' available for use. Result is boolean.
-#   True indicates a successful activation.
+#   'accel' available for use.  True if
+#   successful, and false otherwise.
 #
 # SwitchTo accel --
-#   Activate the implementation named 'accel'.
+#   Activates the implementation named 'accel'.
 #   The empty string disables all implementations.
 
 proc TestAccelInit {namespace} {
@@ -734,13 +746,14 @@ proc TestAccelExit {namespace} {
 # ### ### ### ######### ######### #########
 ##
 
-proc TestFiles {pattern} {
+proc TestFiles pattern {
+    set {local directory} [uplevel 1 [list [namespace which localDirectory]]]
     if {[package vsatisfies [package provide Tcl] 8.3]} {
 	# 8.3+ -directory ok
-	set flist [glob -nocomplain -directory $::tcltest::testsDirectory $pattern]
+	set flist [glob -nocomplain -directory ${local directory} $pattern]
     } else {
 	# 8.2 or less, no -directory
-	set flist [glob -nocomplain [file join $::tcltest::testsDirectory $pattern]]
+	set flist [glob -nocomplain [file join ${local directory} $pattern]]
     }
     foreach f [lsort -dict $flist] {
 	uplevel 1 [list source $f]
@@ -748,13 +761,14 @@ proc TestFiles {pattern} {
     return
 }
 
-proc TestFilesGlob {pattern} {
+proc TestFilesGlob pattern {
+    set {local directory} [uplevel 1 [list [namespace which localDirectory]]]
     if {[package vsatisfies [package provide Tcl] 8.3]} {
 	# 8.3+ -directory ok
-	set flist [glob -nocomplain -directory $::tcltest::testsDirectory $pattern]
+	set flist [glob -nocomplain -directory ${local directory} $pattern]
     } else {
 	# 8.2 or less, no -directory
-	set flist [glob -nocomplain [file join $::tcltest::testsDirectory $pattern]]
+	set flist [glob -nocomplain [file join ${local directory} $pattern]]
     }
     return [lsort -dict $flist]
 }
