@@ -8,12 +8,10 @@
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-# 
-# RCS: @(#) $Id: fileutil.tcl,v 1.78 2010/06/17 04:46:19 andreas_kupries Exp $
 
 package require Tcl 8.2
 package require cmdline
-package provide fileutil 1.14.10
+package provide fileutil 1.16.1
 
 namespace eval ::fileutil {
     namespace export \
@@ -21,7 +19,7 @@ namespace eval ::fileutil {
 	    jail stripPwd stripN stripPath tempdir tempfile \
 	    install fileType writeFile appendToFile \
 	    insertIntoFile removeFromFile replaceInFile \
-	    updateInPlace test tempdirReset
+	    updateInPlace test tempdirReset maketempdir
 }
 
 # ::fileutil::grep --
@@ -626,8 +624,8 @@ if {[string equal $tcl_platform(platform) windows]} {
 
 # ::fileutil::jail --
 #
-#	Ensures that the input path 'filename' stays within the the
-#	directory 'jail'. In this way it preventsuser-supplied paths
+#	Ensures that the input path 'filename' stays within the
+#	directory 'jail'. In this way it prevents user-supplied paths
 #	from escaping the jail.
 #
 # Arguments:
@@ -1420,25 +1418,24 @@ proc ::fileutil::foreachLine {var filename cmd} {
     catch {
 	set code 0
 	set result {}
+	set return 0
 	while {[gets $fp line] >= 0} {
-	    set code [catch {uplevel 1 $cmd} result]
-	    if {($code != 0) && ($code != 4)} {break}
+	    set code [catch {uplevel 1 $cmd} result options]
+	    if {$code == 2} {
+		set return 1
+		set code [dict get $options -code]
+		break
+	    } elseif {$code != 0 && $code != 4} {
+		break
+	    }
 	}
     }
     close $fp
 
-    if {($code == 0) || ($code == 3) || ($code == 4)} {
-        return $result
+    if {$return || $code == 1 || $code > 4} {
+	return -options $options $result
     }
-    if {$code == 1} {
-        global errorCode errorInfo
-        return \
-		-code      $code      \
-		-errorcode $errorCode \
-		-errorinfo $errorInfo \
-		$result
-    }
-    return -code $code $result
+    return $result
 }
 
 # ::fileutil::touch --
@@ -1469,7 +1466,7 @@ if {[package vsatisfies [package provide Tcl] 8.3]} {
 
     proc ::fileutil::touch {args} {
         # Don't bother catching errors, just let them propagate up
-        
+
         set options {
             {a          "set the atime only"}
             {m          "set the mtime only"}
@@ -1480,12 +1477,12 @@ if {[package vsatisfies [package provide Tcl] 8.3]} {
         set usage ": [lindex [info level 0] 0]\
                       \[options] filename ...\noptions:"
         array set params [::cmdline::getoptions args $options $usage]
-        
+
         # process -a and -m options
         set set_atime [set set_mtime "true"]
         if {  $params(a) && ! $params(m)} {set set_mtime "false"}
         if {! $params(a) &&   $params(m)} {set set_atime "false"}
-        
+
         # process -r and -t
         set has_t [expr {$params(t) != -1}]
         set has_r [expr {[string length $params(r)] > 0}]
@@ -1524,10 +1521,10 @@ if {[package vsatisfies [package provide Tcl] 8.3]} {
 #
 # Results
 #	type            Type of the file.  May be a list if multiple tests
-#                       are positive (eg, a file could be both a directory 
+#                       are positive (eg, a file could be both a directory
 #                       and a link).  In general, the list proceeds from most
 #                       general (eg, binary) to most specific (eg, gif), so
-#                       the full type for a GIF file would be 
+#                       the full type for a GIF file would be
 #                       "binary graphic gif"
 #
 #                       At present, the following types can be detected:
@@ -1548,7 +1545,7 @@ if {[package vsatisfies [package provide Tcl] 8.3]} {
 #                       gravity_wave_data_frame
 #                       link
 #			doctools, doctoc, and docidx documentation files.
-#                  
+#
 
 proc ::fileutil::fileType {filename} {
     ;## existence test
@@ -1629,6 +1626,23 @@ proc ::fileutil::fileType {filename} {
                 lappend type executable dos
             }
         }
+    } elseif { $binary && [string match "SQLite format 3\x00*" $test] } {
+	lappend type sqlite3
+
+	# Check for various sqlite-based application file formats.
+	set appid [string range $test 68 71]
+	if {$appid eq "\x0f\x05\x51\x12"} {
+	    lappend type fossil-checkout
+	} elseif {$appid eq "\x0f\x05\x51\x13"} {
+	    lappend type fossil-global-config
+	} elseif {$appid eq "\x0f\x05\x51\x11"} {
+	    lappend type fossil-repository
+	} else {
+	    # encode the appid as hex and append that.
+	    binary scan $appid H8 aid
+	    lappend type A$aid
+	}
+
     } elseif { $binary && [string match "BZh91AY\&SY*" $test] } {
         lappend type compressed bzip
     } elseif { $binary && [string match "\x1f\x8b*" $test] } {
@@ -1656,10 +1670,10 @@ proc ::fileutil::fileType {filename} {
         lappend type graphic tiff
     } elseif { $binary && [string match "BM*" $test] && [string range $test 6 9] == "\x00\x00\x00\x00" } {
         lappend type graphic bitmap
-    } elseif { $binary && [string match "\%PDF\-*" $test] } {
-        lappend type pdf
     } elseif { ! $binary && [string match -nocase "*\<html\>*" $test] } {
         lappend type html
+    } elseif {[string match "\%PDF\-*" $test] } {
+        lappend type pdf
     } elseif { [string match "\%\!PS\-*" $test] } {
        lappend type ps
        if { [string match "* EPSF\-*" $test] } {
@@ -1864,10 +1878,62 @@ namespace eval ::fileutil {
     variable tempdirSet 0
 }
 
+# ::fileutil::maketempdir --
+
+proc ::fileutil::maketempdir {args} {
+    return [Normalize [MakeTempDir $args]]
+}
+
+proc ::fileutil::MakeTempDir {config} {
+    # Setup of default configuration.
+    array set options {}
+    set options(-suffix) ""
+    set options(-prefix) "tmp"
+    set options(-dir)    [tempdir]
+
+    # TODO: Check for and reject options not in -suffix, -prefix, -dir
+    # Merge user configuration, overwrite defaults.
+    array set options $config
+
+    # See also "tempfile" below. Could be shareable internal configuration.
+    set chars       abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
+    set nrand_chars 10
+    set maxtries    10
+
+    for {set i 0} {$i < $maxtries} {incr i} {
+	# Build up the candidate name. See also "tempfile".
+	set directory_name $options(-prefix)
+	for {set j 0} {$j < $nrand_chars} {incr j} {
+	    append directory_name \
+		[string index $chars [expr {int(rand() * 62)}]]
+	}
+	append directory_name $options(-suffix)
+	set path [file join $options(-dir) $directory_name]
+
+	# Try to create. Try again if already exists, or trouble
+	# with creation and setting of perms.
+	#
+	# Note: The last looks as if it is able to leave partial
+	# directories behind (created, trouble with perms). But
+	# deleting ... Might pull the rug out from somebody else.
+
+	if {[file exists $path]} continue
+	if {[catch {
+	    file mkdir $path
+	    if {$::tcl_platform(platform) eq "unix"} {
+		file attributes $path -permissions 0700
+	    }
+	}]} continue
+
+	return $path
+    }
+    return -code error "Failed to find an unused temporary directory name"
+}
+
 # ::fileutil::tempfile --
 #
 #   generate a temporary file name suitable for writing to
-#   the file name will be unique, writable and will be in the 
+#   the file name will be unique, writable and will be in the
 #   appropriate system specific temp directory
 #   Code taken from http://mini.net/tcl/772 attributed to
 #    Igor Volobouev and anon.
