@@ -15,7 +15,7 @@
 
 package require Tcl 8.2
 package require ncgi
-package provide html 1.4.4
+package provide html 1.5
 
 namespace eval ::html {
 
@@ -481,9 +481,12 @@ proc ::html::getTitle {} {
 # Side Effects:
 #	Stores HTML for the <meta> tag for use later by html::head
 
+# Ref: https://www.w3schools.com/tags/tag_meta.asp
+
 proc ::html::meta {args} {
+    # compatibility command
     variable page
-    ::set html ""
+    append html ""
     ::foreach {name value} $args {
 	append html "<meta name=\"$name\" content=\"[quoteFormValue $value]\">"
     }
@@ -491,6 +494,33 @@ proc ::html::meta {args} {
     return ""
 }
 
+proc ::html::meta_name {args} {
+    variable page
+    append html ""
+    ::foreach {name value} $args {
+	append html "<meta name=\"$name\" content=\"[quoteFormValue $value]\">"
+    }
+    lappend page(meta) $html
+    return ""
+}
+
+proc ::html::meta_charset {charset} {
+    variable page
+    append html "<meta charset=\"[quoteFormValue $charset]\">"
+    lappend page(meta) $html
+    return ""
+}
+
+proc ::html::meta_equiv {args} {
+    variable page
+    append html ""
+    ::foreach {name value} $args {
+	append html "<meta http-equiv=\"$name\" content=\"[quoteFormValue $value]\">"
+    }
+    lappend page(meta) $html
+    return ""
+}
+
 # ::html::refresh
 #
 #	Generate a meta refresh tag.  This tag gets bundled into the <head>
@@ -582,8 +612,8 @@ proc ::html::author {author} {
 # ::html::tagParam
 #
 #	Return a name, value string for the tag parameters.
-#	The values come from "hard-wired" values in the 
-#	param agrument, or from the defaults set with html::init.
+#	The values come from "hard-wired" values in the
+#	param argument, or from the defaults set with html::init.
 #
 # Arguments:
 #	tag	Name of the HTML tag (case insensitive).
@@ -924,14 +954,17 @@ proc ::html::textarea {name {param {}} {current {}}} {
 #
 # Arguments:
 #	label		The string to appear in the submit button.
-#	name		The name for the submit button element
+#	name		The name for the submit button element.
+#	title		The string to appear on the submit button.
+#			Optional. If not specified no title is shown.
 #
 # Results:
 #	The html fragment
 
-
-proc ::html::submit {label {name submit}} {
-    ::set html "<input type=\"submit\" name=\"$name\" value=\"$label\">\n"
+proc ::html::submit {label {name submit} {title {}}} {
+    ::set html "<input type=\"submit\" name=\"$name\" value=\"$label\""
+    ::if {$title != ""} { append html " title=\"$title\"" }
+    append html ">\n"
 }
 
 # ::html::varEmpty --
@@ -1014,6 +1047,30 @@ proc ::html::h {level string {param {}}} {
     return "<[string trimright "h$level [tagParam h$level $param]"]>$string</h$level>\n"
 }
 
+# ::html::wrapTag
+#   Takes an optional text and wraps it in a tag pair, along with
+#   optional attributes for the tag
+#
+# Arguments:
+#   tag      The HTML tag name 
+#   text     Optional text to insert between open/close tag
+#   args     List of optional attributes and values to use for the tag
+#
+# Results:
+#   String with the text wrapped in the open/close tag
+
+proc ::html::wrapTag {tag {text ""} args} {
+    ::set html ""
+    ::set params ""
+    ::foreach {i j} $args {
+        append params "$i=\"[quoteFormValue $j]\" "
+    }
+    append html [openTag $tag [string trimright $params]]
+    append html $text
+    append html [closeTag]
+    return $html
+}
+
 # ::html::openTag
 #	Remember that a tag  is opened so it can be closed later.
 #	This is used to automatically clean up at the end of a page.
@@ -1235,7 +1292,7 @@ proc ::html::mailto {email {subject {}}} {
 #	set via html::init
 #
 # Arguments:
-#	args	Font parameters.  
+#	args	Font parameters.
 #
 # Results:
 #	HTML
@@ -1342,7 +1399,7 @@ proc ::html::extractParam {param key {varName ""}} {
 	upvar $varName result
     }
     ::set ws " \t\n\r"
- 
+
     # look for name=value combinations.  Either (') or (") are valid delimeters
     ::if {
       [regsub -nocase [format {.*%s[%s]*=[%s]*"([^"]*).*} $key $ws $ws] $param {\1} value] ||
@@ -1355,7 +1412,7 @@ proc ::html::extractParam {param key {varName ""}} {
     # now look for valueless names
     # I should strip out name=value pairs, so we don't end up with "name"
     # inside the "value" part of some other key word - some day
-	
+
     ::set bad \[^a-zA-Z\]+
     ::if {[regexp -nocase  "$bad$key$bad" -$param-]} {
 	return 1
@@ -1392,7 +1449,24 @@ proc ::html::urlParent {url} {
 
 proc ::html::html_entities {s} {
     variable entities
-    return [string map $entities $s]
+    ::set text [string map $entities $s]
+    ::if {[string is ascii $text]} {
+        return $text
+    }
+    # Escape unicode characters
+    ::set N [string length $text]
+    ::set c 0
+    ::set result {}
+    ::for {::set x 0} {$x < $N} {::incr x} {
+        ::set char [string index $text $x]
+        ::set code [::scan $char %c]
+        ::if {$code>255} {
+            ::append result "&#$code\;"
+        } else {
+            ::append result $char
+        }
+    }
+    return $result
 }
 
 # ::html::nl2br --
@@ -1504,3 +1578,4 @@ proc ::html::js-clear {} {
     catch { unset page(js) }
     return
 }
+
