@@ -12,14 +12,10 @@
 #
 #
 
-if {$::tcl_version < 8.5} {
-    package require dict
-}
-
-package provide yaml 0.3.7
+package require Tcl 8.5
+package provide yaml 0.4.1
 package require cmdline
-package require huddle
-
+package require huddle 0.1.7
 
 namespace eval ::yaml {
     namespace export load setOptions dict2dump list2dump
@@ -36,7 +32,7 @@ namespace eval ::yaml {
     #   return ""
     # }
     variable parsers
-    
+
     # scalar/collection treatment for matched specific yaml-tag
     # proc some_composer {type value} {
     #   return [list 1 $result-type $treatmented-value]
@@ -45,13 +41,14 @@ namespace eval ::yaml {
     # }
     variable composer
 
-    variable defaults 
+    variable defaults
     array set defaults {
         isfile   0
         validate 0
         types {timestamp int float null true false}
         composer {
             !!binary ::yaml::_composeBinary
+	    !!float  ::yaml::_composeFloat
         }
         parsers {
             timestamp ::yaml::_parseTimestamp
@@ -68,18 +65,19 @@ namespace eval ::yaml {
             false:Group {false off - no n}
         }
     }
-    
+
     variable _dumpIndent   2
     variable _dumpWordWrap 40
 
+    variable result
     variable opts [lrange [::cmdline::GetOptionDefaults {
         {file             {input is filename}}
         {stream           {input is stream}}
-        {m.arg        ""  {fixed-modifiers bulk setting(null/true/false)}}
-        {m:null.arg   ""  {null modifier setting(default {"" {null "" ~}})}}
-        {m:true.arg   ""  {true modifier setting(default {1 {true on + yes y}})}}
-        {m:false.arg  ""  {false modifier setting(default {0 {false off - no n}})}}
-        {types.arg    ""  {modifier list setting(default {nop timestamp integer null true false})}}
+        {m.arg        ""  {fixed-modifiers bulk settings(null/true/false)}}
+        {m:null.arg   ""  {null modifier settings(default {"" {null "" ~}})}}
+        {m:true.arg   ""  {true modifier settings(default {1 {true on + yes y}})}}
+        {m:false.arg  ""  {false modifier settings(default {0 {false off - no n}})}}
+        {types.arg    ""  {modifier list settings(default {nop timestamp integer null true false})}}
         {validate         {to validate the input(not dumped tcl content)}}
     } result] 2 end] ;# Remove ? and help.
 
@@ -99,26 +97,31 @@ namespace eval ::yaml {
     }
 }
 
-
 ####################
 # Public APIs
 ####################
 
 proc ::yaml::yaml2dict {args} {
+    variable data
     _getOption $args
-    
+
     set result [_parseBlockNode]
-    if {$yaml::data(validate)} {
+
+    set a [huddle get_stripped $result]
+
+    if {$data(validate)} {
         set result [string map "{\n} {\\n}" $result]
     }
-    return [huddle strip $result]
+
+    return [huddle get_stripped $result]
 }
 
 proc ::yaml::yaml2huddle {args} {
+    variable data
     _getOption $args
-    
+
     set result [_parseBlockNode]
-    if {$yaml::data(validate)} {
+    if {$data(validate)} {
         set result [string map "{\n} {\\n}" $result]
     }
     return $result
@@ -134,17 +137,17 @@ proc ::yaml::setOptions {argv} {
 #
 
 proc ::yaml::list2yaml {list {indent 2} {wordwrap 40}} {
-    return [huddle2yaml [eval huddle list $list] $indent $wordwrap]
+    return [huddle2yaml [huddle list {*}$list] $indent $wordwrap]
 }
 
 proc ::yaml::dict2yaml {dict {indent 2} {wordwrap 40}} {
-    return [huddle2yaml [eval huddle create $dict] $indent $wordwrap]
+    return [huddle2yaml [huddle create {*}$dict] $indent $wordwrap]
 }
 
 proc ::yaml::huddle2yaml {huddle {indent 2} {wordwrap 40}} {
     set yaml::_dumpIndent   $indent
     set yaml::_dumpWordWrap $wordwrap
-    
+
     # Start at the base of the array and move through it.
     set out [join [list "---\n" [_imp_huddle2yaml $huddle] "\n"] ""]
     return $out
@@ -152,7 +155,7 @@ proc ::yaml::huddle2yaml {huddle {indent 2} {wordwrap 40}} {
 
 
 ####################
-# Option Setting
+# Option settings
 ####################
 
 proc ::yaml::_getOption {argv} {
@@ -161,7 +164,7 @@ proc ::yaml::_getOption {argv} {
     variable fixed
     variable composer
 
-    # default setting
+    # default settings
     array set options [_imp_getOptions argv]
 
     array set fixed    $options(fixed)
@@ -169,7 +172,7 @@ proc ::yaml::_getOption {argv} {
     array set composer $options(composer)
     array set data [list validate $options(validate) types $options(types)]
     set isfile $options(isfile)
-    
+
     foreach {buffer} $argv break
     if {$isfile} {
         set fd [open $buffer r]
@@ -190,7 +193,7 @@ proc ::yaml::_imp_getOptions {{argvvar argv}} {
     variable opts
     array set options [array get defaults]
 
-    # default setting
+    # default settings
     array set fixed $options(fixed)
 
     # parse argv
@@ -235,16 +238,21 @@ proc ::yaml::_imp_getOptions {{argvvar argv}} {
 # Scalar/Block Composers
 #########################
 proc ::yaml::_composeTags {tag value} {
+    variable composer
     if {$tag eq ""} {return $value}
-    set value [huddle strip $value]
+    set value [huddle get_stripped $value]
     if {$tag eq "!!str"} {
         set pair [list $tag $value]
-    } elseif {[info exists yaml::composer($tag)]} {
-        set pair [$yaml::composer($tag) $value]
+    } elseif {[info exists composer($tag)]} {
+        set pair [$composer($tag) $value]
     } else {
         error [_getErrorMessage TAG_NOT_FOUND $tag]
     }
-    return  [eval huddle wrap $pair]
+    return  [huddle wrap $pair]
+}
+
+proc ::yaml::_composeFloat {value} {
+    return [list !!float [expr {double($value)}]]
 }
 
 proc ::yaml::_composeBinary {value} {
@@ -255,19 +263,22 @@ proc ::yaml::_composeBinary {value} {
 proc ::yaml::_composePlain {value} {
     if {$value ne ""} {
         if {[huddle type $value] ne "plain"} {return $value}
-        set value [huddle strip $value]
+        set value [huddle get_stripped $value]
     }
     set pair [_toType $value]
-    return  [eval huddle wrap $pair]
+    return  [huddle wrap $pair]
 }
 
 proc ::yaml::_toType {value} {
+    variable data
+    variable parsers
+    variable fixed
     if {$value eq ""} {return [list !!str ""]}
-    
+
     set lowerval [string tolower $value]
-    foreach {type} $yaml::data(types) {
-        if {[info exists yaml::parsers($type)]} {
-            set pair [$yaml::parsers($type) $value]
+    foreach {type} $data(types) {
+        if {[info exists parsers($type)]} {
+            set pair [$parsers($type) $value]
             if {$pair ne ""} {return $pair}
             continue
         }
@@ -288,9 +299,9 @@ proc ::yaml::_toType {value} {
             }
             default {
                 # !!null !!true !!false
-                if {[info exists yaml::fixed($type:Group)] \
-                 && [lsearch $yaml::fixed($type:Group) $lowerval] >= 0} {
-                    set value $yaml::fixed($type:Value)
+                if {[info exists fixed($type:Group)] \
+                 && [lsearch $fixed($type:Group) $lowerval] >= 0} {
+                    set value $fixed($type:Value)
                     return [list !!$type $value]
                 }
             }
@@ -325,11 +336,11 @@ proc ::yaml::_parseBlockNode {{status ""} {indent -1}} {
                 continue
             } else {
                 _ungetc 2
-                
+
                 # [Spec]
-                # Since people perceive theg-hindicator as indentation, 
-                # nested block sequences may be indented by one less space 
-                # to compensate, except, of course, 
+                # Since people perceive the "-" indicator as indentation,
+                # nested block sequences may be indented by one less space
+                # to compensate, except, of course,
                 # if nested inside another block sequence.
                 incr current
             }
@@ -341,11 +352,11 @@ proc ::yaml::_parseBlockNode {{status ""} {indent -1}} {
                 break
             } else {
                 _ungetc 2
-                
+
 #                 # [Spec]
-#                 # Since people perceive theg-hindicator as indentation, 
-#                 # nested block sequences may be indented by one less space 
-#                 # to compensate, except, of course, 
+#                 # Since people perceive the "-" indicator as indentation,
+#                 # nested block sequences may be indented by one less space
+#                 # to compensate, except, of course,
 #                 # if nested inside another block sequence.
 #                 incr current
             }
@@ -393,7 +404,7 @@ proc ::yaml::_parseBlockNode {{status ""} {indent -1}} {
             }
             "*" { ; # alias node
                 set alias [_getToken]
-                if {$yaml::data(validate)} {
+                if {$data(validate)} {
                     set status "ALIAS"
                     set value *$alias
                 } else {
@@ -432,7 +443,7 @@ proc ::yaml::_parseBlockNode {{status ""} {indent -1}} {
         }
     }
     if {$status eq "SEQUENCE"} {
-        set result [eval huddle sequence $result]
+        set result [huddle sequence {*}$result]
     } elseif {$status eq "MAPPING"} {
         if {[llength $prev] == 2} {
             set result [_set_huddle_mapping $result $prev]
@@ -444,7 +455,7 @@ proc ::yaml::_parseBlockNode {{status ""} {indent -1}} {
         set result [lindex $result 0]
         set result [_composePlain $result]
         if {![huddle isHuddle $result]} {
-            set result [huddle wrap !!str $result]
+            set result [huddle wrap [list !!str $result]]
         }
     }
     if {$tag ne ""} {
@@ -467,20 +478,21 @@ proc ::yaml::_mergeExpandedAliases {result pos prev} {
     }
 
     set value [_parseBlockNode "" $pos]
-    if {[huddle type $value] eq "list"} {
+    set type_name [huddle type $value]
+
+    if {$type_name eq "list" || $type_name  eq "sequence"} {
         set len [huddle llength $value]
         for {set i 0} {$i < $len} {incr i} {
             set sub [huddle get $value $i]
             set result [huddle combine $result $sub]
         }
-        unset sub len
+
     } else {
         set result [huddle combine $result $value]
     }
     return [list $result $prev]
 }
 
-
 proc ::yaml::_parseSubBlock {pos statusnew} {
     upvar 1 status status
     set scalar 0
@@ -498,11 +510,15 @@ proc ::yaml::_parseSubBlock {pos statusnew} {
 }
 
 proc ::yaml::_set_huddle_mapping {result prev} {
+
     foreach {key val} $prev break
+
     set val [_composePlain $val]
     if {[huddle isHuddle $key]} {
-        set key [huddle strip $key]
+        set key [huddle get_stripped $key]
     }
+
+
     if {$result eq ""} {
         set result [huddle mapping $key $val]
     } else {
@@ -529,7 +545,7 @@ proc ::yaml::_remove_duplication {dict} {
 # folding ">" (line separator is " ")
 proc ::yaml::_parseBlockScalar {base separator} {
     foreach {explicit chomping} [_parseBlockIndicator] break
-    
+
     set idch [string repeat " " $explicit]
     set sep $separator
     foreach {indent c line} [_getLine] break
@@ -538,7 +554,7 @@ proc ::yaml::_parseBlockScalar {base separator} {
     set first $indent
     set value $line
     set stop 0
-    
+
     while {![_eof]} {
         set pos [_getpos]
         foreach {indent c line} [_getLine] break
@@ -568,8 +584,11 @@ proc ::yaml::_parseBlockScalar {base separator} {
         "clip" {
             append value "\n"
         }
+	default {
+	    error "Should not be reached (chomping = $chomping)"
+	}
     }
-    return [huddle wrap !!str $value]
+    return [huddle wrap [list !!str $value]]
 }
 
 # in {> |}
@@ -617,7 +636,7 @@ proc ::yaml::_parsePlainScalarInBlock {base {loop 0}} {
             append lb "\n"
         }
         set lb [string range $lb 1 end]
-        if {!$yaml::data(finished)} {
+        if {!$data(finished)} {
             _setpos $fpos
         }
         if {$start == $data(start)} {
@@ -677,7 +696,7 @@ proc ::yaml::_parseFlowNode {{status ""}} {
             }
             "\]" { ; # ends a flow sequence
                 if {$status ne "SEQUENCE"} {error [_getErrorMessage SEQEND_NOT_IN_SEQ] }
-                set result [eval huddle sequence $result]
+                set result [huddle sequence {*}$result]
                 return $result
             }
             "&" { ; # node's anchor property
@@ -769,7 +788,7 @@ proc ::yaml::_parseScalarNode {type scope {pos 0}} {
             set tag !!plain
         }
     }
-    return [huddle wrap $tag $value]
+    return [huddle wrap [list $tag $value]]
 }
 
 # [time scanning at JST]
@@ -831,7 +850,7 @@ proc ::yaml::_parseDirective {} {
     variable shorthands
 
     set directive [_getToken]
-    
+
     if {[regexp {^%YAML} $directive]} {
         # YAML directive
         _skipSpaces
@@ -853,7 +872,7 @@ proc ::yaml::_parseDirective {} {
 
 proc ::yaml::_parseTagHandle {} {
     set token [_getToken]
-    
+
     if {[regexp {^(!|!\w*!)(.*)} $token nop handle named]} {
         # shorthand or non-specific Tags
         switch -- $handle {
@@ -862,7 +881,7 @@ proc ::yaml::_parseTagHandle {} {
             !! { ;      # yaml Tags
             }
             default { ; # shorthand Tags
-                
+
             }
         }
         if {![info exists prefix($handle)]} { error [_getErrorMessage TAG_NOT_FOUND] }
@@ -872,7 +891,7 @@ proc ::yaml::_parseTagHandle {} {
     } else {
         error [_getErrorMessage ILLEGAL_TAG_HANDLE]
     }
-    
+
     return "!<$prefix($handle)$named>"
 }
 
@@ -909,7 +928,7 @@ proc ::yaml::_parseSingleQuoted {} {
     regsub -all { ?\r} $result "\n" result
 
     regsub -all {''} [string range $result 1 end-1] {'} chopped
-    
+
     return $chopped
 }
 
@@ -938,7 +957,7 @@ proc ::yaml::_getFoldedString {reStr} {
     set buff [string range $data(buffer) $data(start) end]
     regexp $reStr $buff token
     if {![info exists token]} {return}
-    
+
     set len [string length $token]
     if {[string first "\n" $token] >= 0} { ; # multi-line
         set data(current) [expr {$len - [string last "\n" $token]}]
@@ -946,7 +965,7 @@ proc ::yaml::_getFoldedString {reStr} {
         incr data(current) $len
     }
     incr data(start) $len
-    
+
     return $token
 }
 
@@ -980,6 +999,9 @@ proc ::yaml::_skipSpaces {{commentSkip 0}} {
                     continue
                 }
             }
+	    default {
+		# Any other character, do nothing
+	    }
         }
         break
     }
@@ -1084,11 +1106,12 @@ proc ::yaml::_getAnchor {anchor} {
 }
 
 proc ::yaml::_getErrorMessage {ID {p1 ""}} {
+    variable errors
     set num [_getLineNum]
     if {$p1 != ""} {
-        return "line($num): [subst -nobackslashes -nocommands $yaml::errors($ID)]"
+        return "line($num): [subst -nobackslashes -nocommands $errors($ID)]"
     } else {
-        return "line($num): $yaml::errors($ID)"
+        return "line($num): $errors($ID)"
     }
 }
 
@@ -1104,10 +1127,11 @@ proc ::yaml::_getIndent {line} {
 ################
 
 proc ::yaml::_imp_huddle2yaml {data {offset ""}} {
-    set nextoff "$offset[string repeat { } $yaml::_dumpIndent]"
+    variable _dumpIndent
+    set nextoff "$offset[string repeat { } $_dumpIndent]"
     switch -- [huddle type $data] {
         "string" {
-            set data [huddle strip $data]
+            set data [huddle get_stripped $data]
             return [_dumpScalar $data $offset]
         }
         "list" {
@@ -1169,7 +1193,7 @@ proc ::yaml::_doFolding {value offset} {
     if {$_dumpWordWrap == 0} {
         return $value
     }
-    
+
     if {[string length $value] > $_dumpWordWrap} {
         set wrapped [_simple_justify $value $_dumpWordWrap "\n$offset"]
         set value ">\n$offset$wrapped"
@@ -1198,90 +1222,76 @@ proc ::yaml::_simple_justify {text width {wrap \n} {cut 0}} {
 }
 
 ########################
-## Huddle Settings    ##
+##    YAML TYPES      ##
 ########################
 
+namespace eval ::yaml::types {
+    namespace eval mapping {
+    variable settings
+        set settings {
+        superclass dict
+        publicMethods {mapping}
+        tag !!map
+        isContainer yes }
 
-proc ::yaml::_huddle_mapping {command args} {
-    switch -- $command {
-        setting { ; # type definition
-            return {
-                type dict
-                method {mapping}
-                tag {!!map parent}
-                constructor mapping
-                str !!str
-            }
-        }
-        mapping { ; # $args: all arguments after "huddle mapping"
+        proc mapping {args} {
             if {[llength $args] % 2} {error {wrong # args: should be "huddle mapping ?key value ...?"}}
             set resultL {}
             foreach {key value} $args {
-                lappend resultL $key [huddle to_node $value !!str]
+                lappend resultL $key [argument_to_node $value !!str]
             }
-            return [huddle wrap !!map $resultL]
+            return [huddle wrap [list !!map $resultL]]
         }
-        default { ; # devolving to default dict-callback
-            return [huddle call D $command $args]
-        }
-    }
-}
 
-proc ::yaml::_huddle_sequence {command args} {
-    switch -- $command {
-        setting { ; # type definition
-            return {
-                type list
-                method {sequence}
-                tag {!!seq parent}
-                constructor sequence
-                str !!str
-            }
-        }
-        sequence {
+    }
+
+    namespace eval sequence {
+	variable settings
+
+        set settings {
+	    superclass list
+	    publicMethods {sequence}
+	    isContainer yes
+	    tag !!seq
+	}
+
+        proc sequence {args} {
             set resultL {}
             foreach {value} $args {
-                lappend resultL [huddle to_node $value !!str]
+                lappend resultL [argument_to_node $value !!str]
             }
-            return [huddle wrap !!seq $resultL]
-        }
-        default {
-            return [huddle call L $command $args]
+            return [wrap [list !!seq $resultL]]
         }
+
     }
 }
 
 proc ::yaml::_makeChildType {type tag} {
-    set procname ::yaml::_huddle_$type
-    proc $procname {command args} [string map "@TYPE@ $type @TAG@ $tag" {
-        switch -- $command {
-            setting { ; # type definition
-                return {
-                    type @TYPE@
-                    method {}
-                    tag {@TAG@ child}
-                    constructor ""
-                    str @TAG@
-                }
-            }
-            default {
-                return [huddle call s $command $args]
-            }
-        }
+    set full_path_to_type ::yaml::types::$type
+    namespace eval $full_path_to_type [string map [list @TYPE@ $type @TAG@ $tag] {
+	variable settings
+	set settings {
+	    superClass string
+	    publicMethods {}
+	    isContainer no
+	    tag @TAG@
+	}
     }]
-    return $procname
+
+    return $full_path_to_type
 }
 
-huddle addType ::yaml::_huddle_mapping
-huddle addType ::yaml::_huddle_sequence
-huddle addType [::yaml::_makeChildType string !!str]
-huddle addType [::yaml::_makeChildType string !!timestamp]
-huddle addType [::yaml::_makeChildType string !!float]
-huddle addType [::yaml::_makeChildType string !!int]
-huddle addType [::yaml::_makeChildType string !!null]
-huddle addType [::yaml::_makeChildType string !!true]
-huddle addType [::yaml::_makeChildType string !!false]
-huddle addType [::yaml::_makeChildType string !!binary]
+huddle addType ::yaml::types::mapping
+huddle addType ::yaml::types::sequence
+
+huddle addType [::yaml::_makeChildType str !!str]
+huddle addType [::yaml::_makeChildType timestamp !!timestamp]
+huddle addType [::yaml::_makeChildType float !!float]
+huddle addType [::yaml::_makeChildType int !!int]
+huddle addType [::yaml::_makeChildType null !!null]
+huddle addType [::yaml::_makeChildType true !!true]
+huddle addType [::yaml::_makeChildType false !!false]
+huddle addType [::yaml::_makeChildType binary !!binary]
 huddle addType [::yaml::_makeChildType plain !!plain]
 
 
